@@ -1,23 +1,27 @@
 rbind.zoo <- function(..., deparse.level = 1)
 {  
-  args <- list(...)
-  indexes <- do.call("c", lapply(args, index))
+
+  args <- Filter(Negate(is.null), list(...))
+  indexes <- do.call("c", unname(lapply(args, index)))
 
   my.table <- function(x) {
     x <- x[ORDER(x)]
     table(MATCH(x,x))
   }
-  if(max(my.table(indexes)) > 1) stop("indexes overlap")
+  if(max(my.table(indexes)) > 1L) stop("indexes overlap")
 
-  ncols <- sapply(args, NCOL)  
-  if(!all(ncols == ncols[1])) stop("number of columns differ")
+  if(any(sapply(args, function(x) is.null(dim(x)) && length(x) == 0L && length(index(x)) > 0L)))
+    stop("zero-length vectors with non-zero-length index are not allowed")  
 
+  ncols <- sapply(args, NCOL)
+  if(!all(ncols == ncols[1L])) stop("number of columns differ")
+  
   ## process colnames() if any
   nams <- lapply(args, colnames)
   namsNULL <- sapply(nams, is.null)
   if(all(namsNULL)) namsOK <- TRUE else {
-    if(sum(namsNULL) > 0) namsOK <- FALSE else {
-      nam1 <- nams[[1]]
+    if(sum(namsNULL) > 0L) namsOK <- FALSE else {
+      nam1 <- nams[[1L]]
       namsID <- sapply(nams, function(x) identical(x, nam1))
       if(all(namsID)) namsOK <- TRUE else {
         namsSORT <- sapply(nams, function(x) identical(sort(x), sort(nam1)))
@@ -30,15 +34,31 @@ rbind.zoo <- function(..., deparse.level = 1)
   }
   if(!namsOK) warning("column names differ")
 
-  if(ncols[1] > 1)
-    rval <- zoo(do.call("rbind", lapply(args, coredata)), indexes)
+  ## collect data
+  argsdata <- lapply(args, coredata)
+
+  ## (special case: rbinding of vectors with one-column matrices)
+  nulldim <- sapply(argsdata, function(a) is.null(dim(a)))
+  if(ncols[1L] == 1L) {
+    if(nulldim[1] & any(!nulldim)) {
+      argsdata <- lapply(argsdata, function(a) if(is.null(dim(a))) a else a[,1, drop = TRUE])
+      nulldim <- rep(TRUE, length(nulldim))
+    }
+    if(!nulldim[1] & any(nulldim)) {
+      argsdata <- lapply(argsdata, function(a) if(is.null(dim(a))) as.matrix(a) else a)
+      nulldim <- rep(FALSE, length(nulldim))
+    }
+  }
+
+  if((ncols[1L] > 1L) | !all(nulldim))
+    rval <- zoo(do.call("rbind", argsdata), indexes)
   else
-    rval <- zoo(do.call("c", lapply(args, coredata)), indexes)
+    rval <- zoo(do.call("c", argsdata), indexes)
 
   freq <- if(!("zooreg" %in% unlist(sapply(args, class)))) NULL
             else {
 	      freq <- c(frequency(rval), unlist(sapply(args, frequency)))
-	      if((length(freq) == (length(args)+1)) && 
+	      if((length(freq) == (length(args)+1L)) && 
 	         identical(all.equal(max(freq)/freq, round(max(freq)/freq)), TRUE))
 		 max(freq) else NULL
 	    }
@@ -53,23 +73,24 @@ c.zoo <- function(...) {
     rbind.zoo(...)
 }
 
-cbind.zoo <- function(..., all = TRUE, fill = NA, suffixes = NULL)
+cbind.zoo <- function(..., all = TRUE, fill = NA, suffixes = NULL, drop = FALSE)
 {
-  merge.zoo(..., all = all, fill = fill, suffixes = suffixes, retclass = "zoo")
+  merge.zoo(..., all = all, fill = fill, suffixes = suffixes, retclass = "zoo", drop = drop)
 }
 
-merge.zoo <- function(..., all = TRUE, fill = NA, suffixes = NULL, retclass = c("zoo", "list", "data.frame"))
+
+merge.zoo <- function(..., all = TRUE, fill = NA, suffixes = NULL, check.names = FALSE, retclass = c("zoo", "list", "data.frame"), drop = TRUE)
 {
     if (!is.null(retclass)) retclass <- match.arg(retclass)
     # cl are calls to the args and args is a list of the arguments
     cl <- as.list(match.call())
-    cl[[1]] <- cl$all <- cl$fill <- cl$retclass <- cl$suffixes <- NULL
+    cl[[1]] <- cl$all <- cl$fill <- cl$retclass <- cl$suffixes <- cl$check.names <- cl$drop <- NULL
     args <- list(...)
 
     parent <- parent.frame()
 
     is.plain <- function(x) 
-	all(class(x) %in% c("array", "integer", "numeric", "factor", "matrix"))
+	all(class(x) %in% c("array", "integer", "numeric", "factor", "matrix", "logical"))
 
     is.scalar <- function(x) is.plain(x) && length(x) == 1
 
@@ -80,7 +101,7 @@ merge.zoo <- function(..., all = TRUE, fill = NA, suffixes = NULL, retclass = c(
     scalars <- sapply(args, is.scalar)
 
     if(!is.zoo(args[[1]])) args[[1]] <- as.zoo(args[[1]])
-    for(i in seq(along = args))
+    for(i in seq_along(args))
         if (is.plain(args[[i]]))  
             args[[i]] <- zoo(args[[i]], index(args[[1]]), attr(args[[1]], "frequency"))
 	else if (!is.zoo(args[[i]]))
@@ -101,7 +122,7 @@ merge.zoo <- function(..., all = TRUE, fill = NA, suffixes = NULL, retclass = c(
         makeNames <- function(l) {
             nm <- names(l)
             fixup <- if (is.null(nm)) 
-                seq(along = l)
+                seq_along(l)
             else nm == ""
             dep <- sapply(l[fixup], function(x) deparse(x)[1])
             if (is.null(nm)) 
@@ -135,7 +156,7 @@ merge.zoo <- function(..., all = TRUE, fill = NA, suffixes = NULL, retclass = c(
     # fn to get the unique elements in x, in sorted order, using only
     # [, match, length and order
     sort.unique <- function(x) {
-        x <- x[MATCH(x, x) == seq(length = length(x))]
+        x <- x[MATCH(x, x) == seq_len(length(x))]
         x[ORDER(x)]
     }
 
@@ -145,15 +166,15 @@ merge.zoo <- function(..., all = TRUE, fill = NA, suffixes = NULL, retclass = c(
            x <- x[ORDER(x)]
            table(MATCH(x, x))
 	}
-	union <- do.call("c", list)
-	sort.unique(union)[ my.table(union) == length(list) ]
+	union <- do.call("c", unname(list))
+	sort.unique(union)[which(my.table(union) == length(list))]
     }
     indexintersect <- intersect.list(indexlist)
 
     # get the indexes of the final answer which is the union of
     # all indexes of args corresponding to all=TRUE with the intersection
     # of all indexes
-    indexunion <- do.call("c", indexlist[all])
+    indexunion <- do.call("c", unname(indexlist[all]))
     
     indexes <-  if(is.null(indexunion)) indexintersect
       else sort.unique(c(indexunion, indexintersect))
@@ -162,9 +183,11 @@ merge.zoo <- function(..., all = TRUE, fill = NA, suffixes = NULL, retclass = c(
     # indexes <- sort.unique(c(indexunion, indexintersect))
 
     ## check whether resulting objects still got the same frequency
-    freq <- c(frequency(zoo(,indexes)), freq)
-    freq <- if((length(freq) == 2) && identical(all.equal(max(freq)/freq, round(max(freq)/freq)), TRUE))
-       max(freq) else NULL
+    if(!is.null(freq)) {
+      freq <- c(frequency(zoo(,indexes)), freq)
+      freq <- if((length(freq) == 2) && identical(all.equal(max(freq)/freq, round(max(freq)/freq)), TRUE))
+        max(freq) else NULL
+    }
 
     # the f function does the real work
     # it takes a zoo object, a, and fills in a matrix corresponding to
@@ -239,7 +262,8 @@ merge.zoo <- function(..., all = TRUE, fill = NA, suffixes = NULL, retclass = c(
     # apply f to each arg, put result of doing this on all args in list rval
     # and then cbind that list together to produce the required matrix
     rval <- lapply(args, f, ret.zoo = retclass %in% c("list", "data.frame"))
-    for(i in which(scalars)) rval[[i]] <- rval[[i]][] <- zoo(coredata(rval[[i]])[1], index(rval[[1]]), freq)
+    ## have commented this next line out.  Is it needed?
+    # for(i in which(scalars)) rval[[i]] <- rval[[i]][] <- zoo(coredata(rval[[i]])[1], index(rval[[1]]), freq)
     names(rval) <- suffixes
     if (retclass == "list") { 
 	return(rval)
@@ -272,14 +296,15 @@ merge.zoo <- function(..., all = TRUE, fill = NA, suffixes = NULL, retclass = c(
     rval <- rval[sapply(rval, function(x) length(x) > 0)]
     # if there is more than one non-zero length argument then cbind them
     # Note that cbind will create matrices, even when given a single vector, 
-    # so its important not to use it in the single vector case.
-    rval <- if (length(rval) > 1) 
+    # so - if drop=TRUE - we do not to use it in the single vector case.
+    rval <- if (length(rval) > 1L | (length(rval) == 1L & !drop))
 	do.call("cbind", rval)
-    else
+    else if (length(rval) > 0L)
 	rval[[1]]
     # return if vector since remaining processing is only for column names
-    if (length(dim(rval)) == 0) {
-      rval <- zoo(rval, indexes)
+    if (length(dim(rval)) == 0L) {
+      # fixed bug: coredata was missing
+      rval <- zoo(coredata(rval), indexes)
       attr(rval, "frequency") <- freq
       if(!is.null(freq)) class(rval) <- c("zooreg", class(rval))
       return(rval)
@@ -288,11 +313,12 @@ merge.zoo <- function(..., all = TRUE, fill = NA, suffixes = NULL, retclass = c(
     # processing from here on is to compute nice column names
     if (length(unlist(sapply(args, colnames))) > 0) {
         fixcolnames <- function(a) {
-            if (length(a) == 0) 
-                return(NULL)
+            # if (length(a) == 0) 
+            #   return(NULL)
             if (length(dim(a)) ==0) {
-                return("")
+                if (length(a) == 0) return(NULL) else return("")
             } else {
+				if (ncol(a) == 0) return(NULL)
                 rval <- colnames(a)
                 if (is.null(rval)) {
                   rval <- paste(1:NCOL(a), suffixes[i], sep = ".")
@@ -311,12 +337,11 @@ merge.zoo <- function(..., all = TRUE, fill = NA, suffixes = NULL, retclass = c(
             rval[rval == ""] <- suffixes[i]
             rval
         }
-        zoocolnames <- lapply(seq(along = args), f)
+        zoocolnames <- lapply(seq_along(args), f)
         f <- function(i) ifelse(fixme[[i]], paste(zoocolnames[[i]], 
             suffixes[i], sep = "."), zoocolnames[[i]])
         if (any(duplicated(unlist(zoocolnames)))) 
-            zoocolnames <- lapply(seq(along = args), f)
-        colnames(rval) <- make.unique(unlist(zoocolnames))
+            zoocolnames <- lapply(seq_along(args), f)
     } else {
         fixcolnames <- function(a) {
             if (length(a) == 0) 
@@ -326,12 +351,14 @@ merge.zoo <- function(..., all = TRUE, fill = NA, suffixes = NULL, retclass = c(
             else return(paste(".", 1:NCOL(a), sep = ""))
         }
         zoocolnames <- lapply(args, fixcolnames)
-        zoocolnames <- unlist(lapply(seq(along = args), function(i) 
+        zoocolnames <- lapply(seq_along(args), function(i) 
 		if (!is.null(zoocolnames[[i]])) # NULL returned if false
 			paste(suffixes[i], zoocolnames[[i]], sep = ""))
-	)
-        colnames(rval) <- make.unique(zoocolnames)
     }
+	zoocolnames <- unlist(zoocolnames)
+	colnames(rval) <- if (check.names) make.names(make.unique(zoocolnames))
+		else if (ncol(rval) == length(zoocolnames)) zoocolnames else 
+		colnames(rval)
     # rval <- zoo(rval, indexes)
     rval <- zoo(coredata(rval), indexes)
     attr(rval, "frequency") <- freq
@@ -339,3 +366,4 @@ merge.zoo <- function(..., all = TRUE, fill = NA, suffixes = NULL, retclass = c(
     return(rval)
 }
 
+
